@@ -94,6 +94,8 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|English (British)"), "en_GB", "British English", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
+	{ N_("A-M|English (Canadian)"), "en_CA", "Canadian English", 0 },
+	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|English (Ireland)"), "en_IE", "Irish English", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Estonian"), "et_EE", "Eesti", 0 },
@@ -114,7 +116,7 @@ static Language languages [] = {
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Hebrew"), "iw_IL", "עברית", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
-	{ N_("A-M|Hindi"), "hi_IN", "বাংলা", 0 },
+	{ N_("A-M|Hindi"), "hi_IN", "हिंदी", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
 	{ N_("A-M|Hungarian"), "hu_HU", "Magyar", 0 },
 	/*Note translate the A-M to the A-M you used in the group label */
@@ -168,7 +170,13 @@ static Language languages [] = {
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Russian"), "ru_RU", "русский", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
-	{ N_("N-Z|Serbian"), "sr_YU", "", 0 },
+	{ N_("N-Z|Serbian"), "sr_YU", "српски", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Serbian"), "sr_CS", "српски", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Serbian (Latin)"), "sr_CS@Latn", "srpski (latinica)", 0 },
+	/*Note translate the N-Z to the N-Z you used in the group label */
+	{ N_("N-Z|Serbian (Jekavian)"), "sr_CS@ije", "српски (ијекавски)", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
 	{ N_("N-Z|Slovak"), "sk_SK", "Slovenský", 0 },
 	/*Note translate the N-Z to the N-Z you used in the group label */
@@ -240,7 +248,28 @@ find_lang (const char *language, gboolean *clean)
 
 	p = strrchr (name, '@');
 	if (p != NULL) {
+		char *mod = p+1;
 		*p = '\0';
+
+		/* attempt lookup without encoding but with the
+		   modifier first */
+		p = strrchr (name, '.');
+		if (p != NULL) {
+			char *noenc;
+			*p = '\0';
+
+			noenc = g_strconcat (name, "@", mod, NULL);
+			lang = g_hash_table_lookup (lang_names, noenc);
+			if (lang != NULL) {
+				g_free (name);
+				g_free (noenc);
+				return lang;
+			}
+			g_free (noenc);
+
+			*p = '.';
+		}
+
 		lang = g_hash_table_lookup (lang_names, name);
 		if (lang != NULL) {
 			g_free (name);
