@@ -51,6 +51,7 @@ static gboolean RUNNING_UNDER_GDM = FALSE;
 
 static gboolean gdm_running = FALSE;
 static int GdmMinimalUID = 100;
+static char *GdmSoundProgram = NULL;
 
 static GladeXML *xml;
 
@@ -430,6 +431,52 @@ setup_sensitivity_positive_toggle (const char *name,
 			  depend);
 }
 
+static const char *
+get_root_user (void)
+{
+	static char *root_user = NULL;
+	struct passwd *pwent;
+
+	if (root_user != NULL)
+		return root_user;
+
+	pwent = getpwuid (0);
+	if (pwent == NULL) /* huh? */
+		root_user = g_strdup ("root");
+	else
+		root_user = g_strdup (pwent->pw_name);
+	return root_user;
+}
+
+static void
+root_not_allowed (GtkWidget *entry)
+{
+	static gboolean warned = FALSE;
+	const char *text;
+
+	if (warned)
+		return;
+
+	text = gtk_entry_get_text (GTK_ENTRY (entry));
+
+	if ( ! ve_string_empty (text) &&
+	    strcmp (text, get_root_user ()) == 0) {
+		GtkWidget *dlg = 
+			ve_hig_dialog_new (NULL /* parent */,
+					   GTK_DIALOG_MODAL /* flags */,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   FALSE /* markup */,
+					   _("Autologin or timed login to the superuser (root) account is not allowed."),
+					   /* avoid warning */ "%s", "");
+		if (RUNNING_UNDER_GDM)
+			setup_cursor (GDK_LEFT_PTR);
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
+		warned = TRUE;
+	}
+}
+
 static void
 setup_user_combo (const char *name, const char *key)
 {
@@ -487,6 +534,8 @@ setup_user_combo (const char *name, const char *key)
 
 	g_signal_connect (G_OBJECT (entry), "changed",
 			  G_CALLBACK (entry_changed), NULL);
+	g_signal_connect (G_OBJECT (entry), "changed",
+			  G_CALLBACK (root_not_allowed), NULL);
 
 	g_list_foreach (users, (GFunc)g_free, NULL);
 	g_list_free (users);
@@ -517,6 +566,27 @@ setup_intspin (const char *name,
 			  G_CALLBACK (intspin_changed), NULL);
 }
 
+static void
+setup_notify_entry (const char *name,
+		       const char *key)
+{
+	GtkWidget *entry = glade_helper_get (xml, name, GTK_TYPE_ENTRY);
+	char *val;
+
+	val = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE), key);
+
+	g_object_set_data_full (G_OBJECT (entry),
+				"key", g_strdup (key),
+				(GDestroyNotify) g_free);
+
+	gtk_entry_set_text (GTK_ENTRY (entry), ve_sure_string (val));
+
+	g_signal_connect (G_OBJECT (entry), "changed",
+			  G_CALLBACK (entry_changed), NULL);
+
+	g_free (val);
+}
+
 static gboolean
 greeter_toggle_timeout (GtkWidget *toggle)
 {
@@ -899,6 +969,210 @@ setup_xdmcp_support (void)
 			  xdmcp_frame);
 }
 
+static void
+test_sound (GtkWidget *button, gpointer data)
+{
+	GtkEntry *entry = GTK_ENTRY (data);
+	const char *file = gtk_entry_get_text (GTK_ENTRY (entry));
+	const char *argv[3];
+
+	if (ve_string_empty (file) ||
+	    access (file, R_OK) != 0 ||
+	    ve_string_empty (GdmSoundProgram))
+	       return;
+
+	argv[0] = GdmSoundProgram;
+	argv[1] = file;
+	argv[2] = NULL;
+
+	g_spawn_async ("/" /* working directory */,
+		       (char **)argv,
+		       NULL /* envp */,
+		       0 /* flags */,
+		       NULL /* child setup */,
+		       NULL /* user data */,
+		       NULL /* child pid */,
+		       NULL /* error */);
+}
+
+static gboolean
+module_compare (const char *mod1, const char *mod2)
+{
+	char *base1;
+	char *base2;
+	char *p;
+	gboolean ret;
+
+	/* first cannonify the names */
+	base1 = g_path_get_basename (mod1);
+	base2 = g_path_get_basename (mod2);
+	if (strncmp (base1, "lib", 3) == 0)
+		strcpy (base1, &base1[3]);
+	if (strncmp (base2, "lib", 3) == 0)
+		strcpy (base2, &base2[3]);
+	p = strstr (base1, ".so");
+	if (p != NULL)
+		*p = '\0';
+	p = strstr (base2, ".so");
+	if (p != NULL)
+		*p = '\0';
+
+	ret = (strcmp (base1, base2) == 0);
+
+	g_free (base1);
+	g_free (base2);
+
+	return ret;
+}
+
+static gboolean
+modules_list_contains (const char *modules_list, const char *module)
+{
+	char **vec;
+	int i;
+
+	if (ve_string_empty (modules_list))
+		return FALSE;
+
+	vec = g_strsplit (modules_list, ":", -1);
+	if (vec == NULL)
+		return FALSE;
+
+	for (i = 0; vec[i] != NULL; i++) {
+		if (module_compare (vec[i], module)) {
+			g_strfreev (vec);
+			return TRUE;
+		}
+	}
+
+	g_strfreev (vec);
+	return FALSE;
+}
+
+static char *
+modules_list_remove (char *modules_list, const char *module)
+{
+	char **vec;
+	GString *str;
+	char *sep = "";
+	int i;
+
+	if (ve_string_empty (modules_list))
+		return g_strdup ("");
+
+	vec = g_strsplit (modules_list, ":", -1);
+	if (vec == NULL)
+		return g_strdup ("");
+
+	str = g_string_new (NULL);
+
+	for (i = 0; vec[i] != NULL; i++) {
+		if ( ! module_compare (vec[i], module)) {
+			g_string_append (str, sep);
+			sep = ":";
+			g_string_append (str, vec[i]);
+		}
+	}
+
+	g_strfreev (vec);
+
+	return g_string_free (str, FALSE);
+}
+
+static char *
+modules_list_add (char *modules_list, const char *module)
+{
+	char *n;
+	if (ve_string_empty (modules_list))
+		n = g_strdup (module);
+	else
+		n = g_strconcat (modules_list, ":", module, NULL);
+	g_free (modules_list);
+	return n;
+}
+
+static void
+acc_modules_toggled (GtkWidget *toggle, gpointer data)
+{
+	gboolean add_gtk_modules = ve_config_get_bool (ve_config_get (GDM_CONFIG_FILE),
+						       GDM_KEY_ADD_GTK_MODULES);
+	char *modules_list = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
+						   GDM_KEY_GTK_MODULES_LIST);
+
+	/* first whack the modules from the list */
+	modules_list = modules_list_remove (modules_list, "gail");
+	modules_list = modules_list_remove (modules_list, "atk-bridge");
+	modules_list = modules_list_remove (modules_list, "dwellmouselistener");
+	modules_list = modules_list_remove (modules_list, "keymouselistener");
+
+	if (GTK_TOGGLE_BUTTON (toggle)->active) {
+		if ( ! add_gtk_modules) {
+			g_free (modules_list);
+			modules_list = NULL;
+		}
+
+		modules_list = modules_list_add (modules_list, "gail");
+		modules_list = modules_list_add (modules_list, "atk-bridge");
+		modules_list = modules_list_add (modules_list, EXPANDED_LIBDIR "/gtk-2.0/modules/libkeymouselistener");
+		modules_list = modules_list_add (modules_list, EXPANDED_LIBDIR "/gtk-2.0/modules/libdwellmouselistener");
+
+		add_gtk_modules = TRUE;
+	}
+
+	if (ve_string_empty (modules_list))
+		add_gtk_modules = FALSE;
+
+	ve_config_set_string (ve_config_get (GDM_CONFIG_FILE),
+			      GDM_KEY_GTK_MODULES_LIST,
+			      ve_sure_string (modules_list));
+	ve_config_set_bool (ve_config_get (GDM_CONFIG_FILE),
+			    GDM_KEY_ADD_GTK_MODULES,
+			    add_gtk_modules);
+
+	ve_config_save (ve_config_get (GDM_CONFIG_FILE), FALSE /*force */);
+
+	g_free (modules_list);
+
+	update_key (GDM_KEY_GTK_MODULES_LIST);
+	update_key (GDM_KEY_ADD_GTK_MODULES);
+}
+
+static void
+setup_accessibility_support (void)
+{
+	GtkWidget *acc_modules = glade_helper_get (xml, "acc_modules", GTK_TYPE_TOGGLE_BUTTON);
+	GtkWidget *acc_sound_test = glade_helper_get (xml, "acc_sound_test", GTK_TYPE_BUTTON);
+	GtkWidget *acc_sound_file_entry = glade_helper_get (xml, "acc_sound_file_entry", GTK_TYPE_ENTRY);
+	GtkWidget *acc_sound_file = glade_helper_get (xml, "acc_sound_file", GNOME_TYPE_FILE_ENTRY);
+
+	gboolean add_gtk_modules = ve_config_get_bool (ve_config_get (GDM_CONFIG_FILE),
+						       GDM_KEY_ADD_GTK_MODULES);
+	char *modules_list = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
+						   GDM_KEY_GTK_MODULES_LIST);
+
+	if (add_gtk_modules &&
+	    modules_list_contains (modules_list, "gail") &&
+	    modules_list_contains (modules_list, "atk-bridge") &&
+	    modules_list_contains (modules_list, "dwellmouselistener") &&
+	    modules_list_contains (modules_list, "keymouselistener")) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (acc_modules),
+					      TRUE);
+	} else {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (acc_modules),
+					      FALSE);
+	}
+
+	g_signal_connect (G_OBJECT (acc_modules), "toggled",
+			  G_CALLBACK (acc_modules_toggled),
+			  NULL);
+	g_signal_connect (G_OBJECT (acc_sound_test), "clicked",
+			  G_CALLBACK (test_sound),
+			  acc_sound_file_entry);
+
+	if (access ("/usr/share/sounds", F_OK) == 0)
+		gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (acc_sound_file), "/usr/share/sounds/");
+}
+
 static void
 background_toggled (void)
 {
@@ -1561,10 +1835,9 @@ dir_exists (const char *parent, const char *dir)
 }
 
 static void
-install_ok (GtkWidget *button, gpointer data)
+install_response (GtkWidget *chooser, gint response, gpointer data)
 {
-	GtkFileSelection *fs = GTK_FILE_SELECTION (data);
-	GtkListStore *store = g_object_get_data (G_OBJECT (fs), "ListStore");
+	GtkListStore *store = data;
 	GtkWidget *theme_list = glade_helper_get (xml, "gg_theme_list",
 						  GTK_TYPE_TREE_VIEW);
 	char *filename, *dir, *untar_cmd, *theme_dir, *cwd;
@@ -1574,13 +1847,18 @@ install_ok (GtkWidget *button, gpointer data)
 	DIR *dp;
 	gboolean success = FALSE;
 
+	if (response != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (chooser);
+		return;
+	}
+
 	cwd = g_get_current_dir ();
 	theme_dir = get_theme_dir ();
 
-	filename = g_strdup (gtk_file_selection_get_filename (fs));
+	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
 	if (filename == NULL) {
 		GtkWidget *dlg =
-			ve_hig_dialog_new (GTK_WINDOW (fs),
+			ve_hig_dialog_new (GTK_WINDOW (chooser),
 					   GTK_DIALOG_MODAL | 
 					   GTK_DIALOG_DESTROY_WITH_PARENT,
 					   GTK_MESSAGE_ERROR,
@@ -1608,7 +1886,7 @@ install_ok (GtkWidget *button, gpointer data)
 
 	if (dir == NULL) {
 		GtkWidget *dlg =
-			ve_hig_dialog_new (GTK_WINDOW (fs),
+			ve_hig_dialog_new (GTK_WINDOW (chooser),
 					   GTK_DIALOG_MODAL | 
 					   GTK_DIALOG_DESTROY_WITH_PARENT,
 					   GTK_MESSAGE_ERROR,
@@ -1637,7 +1915,7 @@ install_ok (GtkWidget *button, gpointer data)
 				       "installed, install again anyway?"),
 				     fname);
 		dlg = ve_hig_dialog_new
-			(GTK_WINDOW (fs),
+			(GTK_WINDOW (chooser),
 			 GTK_DIALOG_MODAL | 
 			 GTK_DIALOG_DESTROY_WITH_PARENT,
 			 GTK_MESSAGE_QUESTION,
@@ -1705,7 +1983,7 @@ install_ok (GtkWidget *button, gpointer data)
 
 	if ( ! success) {
 		GtkWidget *dlg =
-			ve_hig_dialog_new (GTK_WINDOW (fs),
+			ve_hig_dialog_new (GTK_WINDOW (chooser),
 					   GTK_DIALOG_MODAL | 
 					   GTK_DIALOG_DESTROY_WITH_PARENT,
 					   GTK_MESSAGE_ERROR,
@@ -1740,36 +2018,36 @@ install_ok (GtkWidget *button, gpointer data)
 	g_free (cwd);
 	g_free (theme_dir);
 
-	gtk_widget_destroy (GTK_WIDGET (fs));
+	gtk_widget_destroy (GTK_WIDGET (chooser));
 }
 
 static void
 install_new_theme (GtkWidget *button, gpointer data)
 {
 	GtkListStore *store = data;
-	static GtkWidget *fs = NULL;
+	static GtkWidget *chooser = NULL;
 	GtkWidget *setup_dialog;
 
-	if (fs != NULL) {
-		gtk_window_present (GTK_WINDOW (fs));
+	if (chooser != NULL) {
+		gtk_window_present (GTK_WINDOW (chooser));
 		return;
 	}
        
 	setup_dialog = glade_helper_get (xml, "setup_dialog", GTK_TYPE_WINDOW);
 	
-	fs = gtk_file_selection_new (_("Select new theme archive to install"));
-	gtk_window_set_transient_for (GTK_WINDOW (fs),
-				      GTK_WINDOW (setup_dialog));
-	g_object_set_data (G_OBJECT (fs), "ListStore", store);
-	g_signal_connect (G_OBJECT (fs), "destroy",
-			  G_CALLBACK (gtk_widget_destroyed), &fs);
-	g_signal_connect (GTK_FILE_SELECTION (fs)->ok_button, "clicked",
-			  G_CALLBACK (install_ok), fs);
-	g_signal_connect_swapped (GTK_FILE_SELECTION (fs)->cancel_button,
-				  "clicked",
-				  G_CALLBACK (gtk_widget_destroy), fs);
+	chooser = gtk_file_chooser_dialog_new (_("Select new theme archive to install"),
+					       GTK_WINDOW (setup_dialog),
+					       GTK_FILE_CHOOSER_ACTION_OPEN,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       _("_Install"), GTK_RESPONSE_OK,
+					       NULL);
+	
+	g_signal_connect (G_OBJECT (chooser), "destroy",
+			  G_CALLBACK (gtk_widget_destroyed), &chooser);
+	g_signal_connect (G_OBJECT (chooser), "response",
+			  G_CALLBACK (install_response), store);
 
-	gtk_widget_show (fs);
+	gtk_widget_show (chooser);
 }
 
 static void
@@ -2096,8 +2374,11 @@ setup_gui (void)
 	glade_helper_tagify_label (xml, "gg_desc_label", "b");
 	glade_helper_tagify_label (xml, "gg_copyright_label", "b");
 
+	glade_helper_tagify_label (xml, "acc_options_cat_label", "b");
+
 	glade_helper_tagify_label (xml, "enable_xdmcp", "b");
 
+	setup_accessibility_support ();
 	setup_xdmcp_support ();
 	setup_background_support ();
 	setup_greeter_backselect ();
@@ -2175,10 +2456,18 @@ setup_gui (void)
 			     GDM_KEY_DISALLOWTCP,
 			     GDM_KEY_DISALLOWTCP /* notify_key */);
 
-	/* setup sensitivities */
+	/* security sensitivities */
 	setup_sensitivity_positive_toggle ("sysmenu", "config_available");
 	setup_sensitivity_positive_toggle ("sysmenu", "chooser_button");
 
+	setup_greeter_toggle ("acc_beep",
+			      GDM_KEY_SOUND_ON_LOGIN);
+	setup_notify_entry ("acc_sound_file_entry",
+			    GDM_KEY_SOUND_ON_LOGIN_FILE);
+
+	/* accesibility sensitivities */
+	setup_sensitivity_positive_toggle ("acc_beep", "acc_sound_file_box");
+
 	setup_notify_toggle ("enable_xdmcp",
 			     GDM_KEY_XDMCP,
 			     GDM_KEY_XDMCP /* notify_key */);
@@ -2381,18 +2670,30 @@ main (int argc, char *argv[])
 
 	if (RUNNING_UNDER_GDM) {
 		char *gtkrc;
+		char *theme_name;
 		guint sid;
 
 		/* Set busy cursor */
 		setup_cursor (GDK_WATCH);
 
-		/* If we are running under gdm parse the GDM gtkRC */
+		/* parse the given gtk rc first */
 		gtkrc = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
 					      GDM_KEY_GTKRC);
 		if ( ! ve_string_empty (gtkrc))
 			gtk_rc_parse (gtkrc);
 		g_free (gtkrc);
 
+		theme_name = g_strdup (g_getenv ("GDM_GTK_THEME"));
+		if (ve_string_empty (theme_name)) {
+			g_free (theme_name);
+			theme_name = ve_config_get_string
+				(ve_config_get (GDM_CONFIG_FILE), GDM_KEY_GTK_THEME);
+		}
+
+		if ( ! ve_string_empty (theme_name)) {
+			gdm_set_theme (theme_name);
+		}
+
 		/* evil, but oh well */
 		g_type_class_ref (GTK_TYPE_WIDGET);	
 
@@ -2433,6 +2734,13 @@ main (int argc, char *argv[])
 	 * ui?  Say it ain't so.  Our config sections are SUCH A MESS */
 	GdmMinimalUID = ve_config_get_int (ve_config_get (GDM_CONFIG_FILE),
 					   GDM_KEY_MINIMALUID);
+	GdmSoundProgram = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
+						GDM_KEY_SOUND_PROGRAM);
+	if (ve_string_empty (GdmSoundProgram) ||
+	    access (GdmSoundProgram, X_OK) != 0) {
+		g_free (GdmSoundProgram);
+		GdmSoundProgram = NULL;
+	}
 
 	setup_gui ();
 
