@@ -30,6 +30,11 @@ extern gboolean GdmAllowRoot;
 extern gboolean GdmAllowRemoteRoot;
 extern gchar *GdmWelcome;
 extern gchar *GdmServAuthDir;
+extern gchar *GdmInfoMsgFile;
+extern gchar *GdmInfoMsgFont;
+extern gchar *GdmSoundProgram;
+extern gchar *GdmSoundOnLoginFile;
+extern gboolean GdmSoundOnLogin;
 
 extern gboolean GDM_IS_LOCAL;
 extern gboolean DOING_GDM_DEVELOPMENT;
