@@ -27,6 +27,7 @@
 #include <gmodule.h>
 #include <gdk/gdk.h>
 #include <gtk/gtk.h>
+#include <vicious.h>
 #include <gnome.h>
 
 #include <config.h>
@@ -418,6 +419,17 @@ load_bindings(gchar *path)
   fclose (fp);
 }
 
+static gboolean
+change_cursor_back (gpointer data)
+{
+  GdkCursor *cursor = gdk_cursor_new (GDK_LEFT_PTR);
+  gdk_window_set_cursor (gdk_get_default_root_window (), cursor);
+  gdk_cursor_unref (cursor);
+
+  return FALSE;
+}
+
+
 static gboolean
 leave_enter_emission_hook (GSignalInvocationHint        *ihint,
                            guint                        n_param_values,
@@ -528,6 +540,7 @@ leave_enter_emission_hook (GSignalInvocationHint        *ihint,
                   for (act_li = curr_binding->actions; act_li != NULL; act_li = act_li->next)
                     {
                       gchar *action = (gchar *)act_li->data;
+		      char *oldpath, *newpath;
                       g_return_val_if_fail (action != NULL, TRUE);
 
                       if (!g_shell_parse_argv (action, NULL, &argv, NULL))
@@ -535,6 +548,18 @@ leave_enter_emission_hook (GSignalInvocationHint        *ihint,
 
                       envp = get_exec_environment (gtk_window_get_screen(window));
 
+		      /* add our BINDIR to the path */
+		      oldpath = g_strdup (g_getenv ("PATH"));
+		      if (ve_string_empty (oldpath))
+		        newpath = g_strdup (EXPANDED_BINDIR);
+		      else 
+		        newpath = g_strconcat (oldpath,
+					       ":",
+					       EXPANDED_BINDIR,
+					       NULL);
+		      ve_setenv ("PATH", newpath, TRUE);
+		      g_free (newpath);
+
                       retval = g_spawn_async (NULL,
                                               argv,
                                               envp,
@@ -543,10 +568,17 @@ leave_enter_emission_hook (GSignalInvocationHint        *ihint,
                                               NULL,
                                               NULL,
                                               NULL);
+
+		      if (ve_string_empty (oldpath))
+		        ve_unsetenv ("PATH");
+		      else
+			ve_setenv ("PATH", oldpath, TRUE);
+		      g_free (oldpath);
+
                       g_strfreev (argv);
                       g_strfreev (envp);
 
-                      if (!retval)
+                      if ( ! retval)
                         {
                           GtkWidget *dialog =
                             gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR,
@@ -561,6 +593,16 @@ leave_enter_emission_hook (GSignalInvocationHint        *ihint,
                                             G_CALLBACK (gtk_widget_destroy),
                                             NULL);
                           gtk_widget_show (dialog);
+			}
+		      else
+		        {
+			  GdkCursor *cursor = gdk_cursor_new (GDK_WATCH);
+			  gdk_window_set_cursor (gdk_get_default_root_window (),
+						 cursor);
+			  gdk_cursor_unref (cursor);
+			  g_timeout_add (2000,
+					 change_cursor_back,
+					 NULL);
                         }
                     }
                 }
