@@ -365,12 +365,22 @@ parse_line (gchar *buf)
 			return NULL;
 		}
 
-		if ((timeout=atoi(tmp_string)) <= 0) {
-			/* Add an error message */;
-			free_gesture (tmp_gesture);
-			return NULL;
+		/*
+		 * A gesture with an n_times value greater than 1 and a
+		 * non-positive timeout can never be triggered, so do not
+		 * accept such gestures.  The value of timeout is not used
+		 * if n_times is 1, so don't bother setting the timeout in
+		 * this case.
+		 */
+		tmp_gesture->timeout = 0;
+		if (tmp_gesture->n_times > 1) {
+			if ((timeout=atoi(tmp_string)) <= 0) {
+				/* Add an error message */;
+				free_gesture (tmp_gesture);
+				return NULL;
+			}
+			tmp_gesture->timeout = timeout;
 		}
-		tmp_gesture->timeout = timeout;
 	}
 
 	/*
@@ -511,16 +521,18 @@ gestures_filter (GdkXEvent *gdk_xevent,
 		}
 
 		/*
-		 * Find the associated gesture for this keycode &state
+		 * Find the associated gesture for this keycode & state
 		 * TODO: write a custom g_slist_find function.
 		 *
-		 * Note that here we don't check the state, otherwise key gestures based on
-		 * modifier keys such as Control_R won't work.
+		 * Note that here we check the state against the last_event,
+		 * otherwise key gestures based on modifier keys such as
+		 * Control_R won't work.
 		 */
 		for (li = gesture_list; li != NULL; li = li->next) {
 			Gesture *gesture = (Gesture *) li->data;
 			if (gesture->type == GESTURE_TYPE_KEY &&
-			    xevent->xkey.keycode == gesture->input.key.keycode) {
+			    xevent->xkey.keycode == gesture->input.key.keycode &&
+			    last_event->xkey.state == gesture->input.key.state) {
 				/* 
 				 * OK Found the gesture.
 				 * Now check if it has a duration value > 0.
@@ -716,7 +728,7 @@ G_MODULE_EXPORT void gtk_module_init(int *argc, char* argv[]);
 void gtk_module_init(int *argc, char* argv[])
 {
 	create_event_watcher();
-}		
+}
 
 /* EOF */
 
